<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <title>Dumb admin ;-)</title>
</head>
<body>

<h2>Unique users remembered: {{ remembered_user_count }}</h2>

<h2>Unique users connected: {{ unique_user_count }}</h2>

<h2>Total connections: {{ total_connections }}</h2>

<h2>Total sys connections: {{ total_sys_connections }}</h2>

<h2>Total channels: {{ total_channels }}</h2>

<h2>Uptime: {{ uptime }}</h2>

<h2>Messages since start: {{ total_unique_messages }}</h2>

<h2>All frames sent: {{ total_messages }}</h2>

<h2>Channels</h2>
<ul>
    {% for channel in channels.values() %}
        <li><strong>{{ channel.name }}</strong>
            - {{ channel.connections |length }} users subscribed<br/>
            <h4>Config</h4>
            <ul>
                <li><strong>Name</strong> --> {{ channel.name }}</li>
                <li><strong>Long Name</strong> --> {{ channel.long_name }}</li>
                <li><strong>notify_presence</strong> --> {{ channel.notify_presence }}</li>
                <li><strong>Salvageable</strong> --> {{ channel.salvageable }}
                </li>
                <li><strong>Store History</strong>
                    --> {{ channel.store_history }}</li>
                <li><strong>History Size</strong> --> {{ channel.history_size }}
                </li>
            </ul>

            <h4>Connections</h4>
            <ul>
                {% for user, conns in channel.connections.items() |sort %}
                    <li><strong>{{ user }}</strong> : {{ conns|length }}
                        connections
                    </li>
                {% endfor %}
            </ul>
        </li>
        <br/>
    {% endfor %}
</ul>

<h2>Users</h2>
<ul>
    {% for user in users.values() %}
    <li><strong>{{ user.username }} - last active {{user.last_active}} - {{user.connections|length}} connections</strong>
        <div>
            <pre>{{user.state}}</pre>
        </div>
    </li>
    {% endfor %}
</ul>

</body>
</html>
